@@ -1,5 +1,5 @@
 /* GAIL - The GNOME Accessibility Implementation Library
- * Copyright 2001 Sun Microsystems Inc.
+ * Copyright 2001, 2002, 2003 Sun Microsystems Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -132,8 +132,6 @@ gail_notebook_new (GtkWidget *widget)
   accessible = ATK_OBJECT (object);
   atk_object_initialize (accessible, widget);
 
-  accessible->role = ATK_ROLE_PAGE_TAB_LIST;
-
   return accessible;
 }
 
@@ -217,6 +215,8 @@ gail_notebook_real_initialize (AtkObject *obj,
   g_object_weak_ref (G_OBJECT(gtk_notebook),
                      (GWeakNotify) gail_notebook_destroyed,
                      obj);                     
+
+  obj->role = ATK_ROLE_PAGE_TAB_LIST;
 }
 
 static void
@@ -273,8 +273,8 @@ gail_notebook_real_notify_gtk (GObject           *obj,
          (focus_page_num != old_focus_page_num))
         {
           if (gail_notebook->idle_focus_id)
-            gtk_idle_remove (gail_notebook->idle_focus_id);
-          gail_notebook->idle_focus_id = gtk_idle_add (gail_notebook_check_focus_tab, atk_obj);
+            g_source_remove (gail_notebook->idle_focus_id);
+          gail_notebook->idle_focus_id = g_idle_add (gail_notebook_check_focus_tab, atk_obj);
         }
     }
   else
@@ -453,8 +453,8 @@ gail_notebook_focus_cb (GtkWidget      *widget,
     case GTK_DIR_LEFT:
     case GTK_DIR_RIGHT:
       if (gail_notebook->idle_focus_id)
-        gtk_idle_remove (gail_notebook->idle_focus_id);
-      gail_notebook->idle_focus_id = gtk_idle_add (gail_notebook_check_focus_tab, atk_obj);
+        g_source_remove (gail_notebook->idle_focus_id);
+      gail_notebook->idle_focus_id = g_idle_add (gail_notebook_check_focus_tab, atk_obj);
       break;
     default:
       break;
@@ -502,5 +502,5 @@ gail_notebook_destroyed (gpointer data)
   GailNotebook *gail_notebook = GAIL_NOTEBOOK (data);
 
   if (gail_notebook->idle_focus_id)
-    gtk_idle_remove (gail_notebook->idle_focus_id);
+    g_source_remove (gail_notebook->idle_focus_id);
 }
