@@ -1,5 +1,5 @@
 /* GAIL - The GNOME Accessibility Implementation Library
- * Copyright 2001 Sun Microsystems Inc.
+ * Copyright 2001, 2002, 2003 Sun Microsystems Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -211,22 +211,6 @@ gail_window_new (GtkWidget *widget)
   accessible = ATK_OBJECT (object);
   atk_object_initialize (accessible, widget);
 
-
-  if (GTK_IS_FILE_SELECTION (widget))
-    accessible->role = ATK_ROLE_FILE_CHOOSER;
-  else if (GTK_IS_COLOR_SELECTION_DIALOG (widget))
-    accessible->role = ATK_ROLE_COLOR_CHOOSER;
-  else if (GTK_IS_FONT_SELECTION_DIALOG (widget))
-    accessible->role = ATK_ROLE_FONT_CHOOSER;
-  else if (GTK_IS_DIALOG (widget))
-    accessible->role = ATK_ROLE_DIALOG;
-  else if (GTK_IS_WINDOW (widget))
-    accessible->role = ATK_ROLE_FRAME;
-  else if (GTK_IS_HANDLE_BOX (widget))
-    accessible->role = ATK_ROLE_UNKNOWN;
-  else
-    accessible->role = ATK_ROLE_INVALID;
-
   return accessible;
 }
 
@@ -234,14 +218,44 @@ static void
 gail_window_real_initialize (AtkObject *obj,
                              gpointer  data)
 {
+  GtkWidget *widget;
+
   ATK_OBJECT_CLASS (parent_class)->initialize (obj, data);
 
+
+  widget = GTK_WIDGET (data);
+
   g_signal_connect (data,
                     "window_state_event",
                     G_CALLBACK (gail_window_state_event_gtk),
                     NULL);
   g_object_set_data (G_OBJECT (obj), "atk-component-layer",
                      GINT_TO_POINTER (ATK_LAYER_WINDOW));
+
+  if (GTK_IS_FILE_SELECTION (widget))
+    obj->role = ATK_ROLE_FILE_CHOOSER;
+  else if (GTK_IS_COLOR_SELECTION_DIALOG (widget))
+    obj->role = ATK_ROLE_COLOR_CHOOSER;
+  else if (GTK_IS_FONT_SELECTION_DIALOG (widget))
+    obj->role = ATK_ROLE_FONT_CHOOSER;
+  else if (GTK_IS_DIALOG (widget))
+    obj->role = ATK_ROLE_DIALOG;
+  else if (GTK_IS_WINDOW (widget))
+    {
+      const gchar *name;
+
+      name = gtk_widget_get_name (widget);
+      if (name && !strcmp (name, "gtk-tooltips"))
+        obj->role = ATK_ROLE_TOOL_TIP;
+      else if (GTK_IS_PLUG (widget))
+        obj->role = ATK_ROLE_PANEL;
+      else
+        obj->role = ATK_ROLE_FRAME;
+    }
+  else if (GTK_IS_HANDLE_BOX (widget))
+    obj->role = ATK_ROLE_UNKNOWN;
+  else
+    obj->role = ATK_ROLE_INVALID;
 }
 
 static G_CONST_RETURN gchar*
@@ -273,7 +287,18 @@ gail_window_get_name (AtkObject *accessible)
           name = gtk_window_get_title (window);
           if (name)
             return name;
+
+          if (accessible->role == ATK_ROLE_TOOL_TIP)
+            {
+              GtkWidget *child;
+
+              child = gtk_bin_get_child (GTK_BIN (window));
+              g_return_val_if_fail (GTK_IS_LABEL (child), NULL);
+              name = gtk_label_get_text (GTK_LABEL (child));
+              return name;
+            }
         }
+                    
       return ATK_OBJECT_CLASS (parent_class)->get_name (accessible);
     }
 }
@@ -285,9 +310,6 @@ gail_window_get_parent (AtkObject *accessible)
 
   parent = ATK_OBJECT_CLASS (parent_class)->get_parent (accessible);
 
-  if (!parent)
-    parent = atk_get_root ();
-
   return parent;
 }
 
@@ -682,15 +704,21 @@ filter_func (GdkXEvent *gdkxevent,
       if (xevent->xproperty.atom == _net_client_list_stacking)
         {
           int     screen_n;
+          GdkWindow *window;
 
-          screen_n = gdk_screen_get_number (
-		gdk_drawable_get_screen (GDK_DRAWABLE (event->any.window)));
+          window = event->any.window;
 
-          gail_screens [screen_n].update_stacked_windows = TRUE;
-          if (!gail_screens [screen_n].update_handler)
+          if (window)
             {
-              gail_screens [screen_n].update_handler = g_idle_add (update_screen_info,
-	        						   GINT_TO_POINTER (screen_n));
+              screen_n = gdk_screen_get_number (
+                  gdk_drawable_get_screen (GDK_DRAWABLE (window)));
+
+              gail_screens [screen_n].update_stacked_windows = TRUE;
+              if (!gail_screens [screen_n].update_handler)
+                {
+                  gail_screens [screen_n].update_handler = g_idle_add (update_screen_info,
+	        						       GINT_TO_POINTER (screen_n));
+                }
             }
         }
       else if (xevent->xproperty.atom == _net_wm_desktop)
